/*
** Module   : Icollection.java
** Abstract : Implementation of the builtin class.
**
** Copyright (c) 2019-2021, Golden Code Development Corporation.
**
** -#- -I- --Date-- ---------------------------------------Description----------------------------------------
** 001 CA  20190526 First version, stubs taken by converting the skeleton using FWD.
**     RFB 20191022 Added LegacyResource annotation
** 002 CA  20191024 Added method support levels and updated the class support level.
** 003 ME  20200729 Update parameter names to match skeleton.
** 004 CA  20210113 Renamed clear() to clear_(), to follow NameConverter's rules.
** 005 CA  20210221 Fixed 'qualified', 'extent' and 'returns' annotations at the legacy signature.
**     GES 20210501 Upgraded table-handle parameter processing to match API changes.
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You may find a copy of the GNU Affero GPL version 3 at the following
** location: https://www.gnu.org/licenses/agpl-3.0.en.html
** 
** Additional terms under GNU Affero GPL version 3 section 7:
** 
**   Under Section 7 of the GNU Affero GPL version 3, the following additional
**   terms apply to the works covered under the License.  These additional terms
**   are non-permissive additional terms allowed under Section 7 of the GNU
**   Affero GPL version 3 and may not be removed by you.
** 
**   0. Attribution Requirement.
** 
**     You must preserve all legal notices or author attributions in the covered
**     work or Appropriate Legal Notices displayed by works containing the covered
**     work.  You may not remove from the covered work any author or developer
**     credit already included within the covered work.
** 
**   1. No License To Use Trademarks.
** 
**     This license does not grant any license or rights to use the trademarks
**     Golden Code, FWD, any Golden Code or FWD logo, or any other trademarks
**     of Golden Code Development Corporation. You are not authorized to use the
**     name Golden Code, FWD, or the names of any author or contributor, for
**     publicity purposes without written authorization.
** 
**   2. No Misrepresentation of Affiliation.
** 
**     You may not represent yourself as Golden Code Development Corporation or FWD.
** 
**     You may not represent yourself for publicity purposes as associated with
**     Golden Code Development Corporation, FWD, or any author or contributor to
**     the covered work, without written authorization.
** 
**   3. No Misrepresentation of Source or Origin.
** 
**     You may not represent the covered work as solely your work.  All modified
**     versions of the covered work must be marked in a reasonable way to make it
**     clear that the modified work is not originating from Golden Code Development
**     Corporation or FWD.  All modified versions must contain the notices of
**     attribution required in this license.
*/

package com.goldencode.p2j.oo.core.collections;

import com.goldencode.p2j.util.*;

import static com.goldencode.p2j.report.ReportConstants.*;
import static com.goldencode.p2j.util.InternalEntry.Type;

/**
 * Business logic (converted to Java from the 4GL source code
 * in OpenEdge/Core/Collections/ICollection.cls).
 */
@LegacyResource(resource = "OpenEdge.Core.Collections.ICollection")
@LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
public interface Icollection
extends Iiterable
{
   @LegacySignature(returns = "INTEGER", type = Type.GETTER, name = "Size")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public integer getSize();

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "Add", parameters = 
   {
      @LegacyParameter(name = "poElement", type = "OBJECT", qualified = "progress.lang.object", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public logical add(final object<? extends com.goldencode.p2j.oo.lang._BaseObject_> _poElement);

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "AddAll", parameters = 
   {
      @LegacyParameter(name = "poCollection", type = "OBJECT", qualified = "openedge.core.collections.icollection", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public logical addAll(final object<? extends com.goldencode.p2j.oo.core.collections.Icollection> _poCollection);

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "AddArray", parameters = 
   {
      @LegacyParameter(name = "poElements", type = "OBJECT", extent = -1, qualified = "progress.lang.object", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public logical addArray(final object<? extends com.goldencode.p2j.oo.lang._BaseObject_>[] _poElements);

   @LegacySignature(type = Type.METHOD, name = "Clear")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public void clear_();

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "Contains", parameters = 
   {
      @LegacyParameter(name = "poElement", type = "OBJECT", qualified = "progress.lang.object", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public logical contains(final object<? extends com.goldencode.p2j.oo.lang._BaseObject_> _poElement);

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "ContainsAll", parameters = 
   {
      @LegacyParameter(name = "poCollection", type = "OBJECT", qualified = "openedge.core.collections.icollection", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public logical containsAll(final object<? extends com.goldencode.p2j.oo.core.collections.Icollection> _poCollection);

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "IsEmpty")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public logical isEmpty();

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "Remove", parameters = 
   {
      @LegacyParameter(name = "poElement", type = "OBJECT", qualified = "progress.lang.object", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public logical remove(final object<? extends com.goldencode.p2j.oo.lang._BaseObject_> _poElement);

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "RemoveAll", parameters = 
   {
      @LegacyParameter(name = "poCollection", type = "OBJECT", qualified = "openedge.core.collections.icollection", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public logical removeAll(final object<? extends com.goldencode.p2j.oo.core.collections.Icollection> _poCollection);

   @LegacySignature(returns = "LOGICAL", type = Type.METHOD, name = "RetainAll", parameters = 
   {
      @LegacyParameter(name = "poCollection", type = "OBJECT", qualified = "openedge.core.collections.icollection", mode = "INPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public logical retainAll(final object<? extends com.goldencode.p2j.oo.core.collections.Icollection> _poCollection);

   @LegacySignature(returns = "OBJECT", type = Type.METHOD, name = "ToArray", extent = -1, qualified = "progress.lang.object")
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public object<? extends com.goldencode.p2j.oo.lang._BaseObject_>[] toArray();

   @LegacySignature(type = Type.METHOD, name = "ToTable", parameters = 
   {
      @LegacyParameter(name = "p1", type = "HANDLE", mode = "OUTPUT")
   })
   @LegacyResourceSupport(supportLvl = CVT_LVL_FULL|RT_LVL_FULL)
   public void toTable(final OutputTableHandle _p1);
}
